from twisted.internet.defer import inlineCallbacks

from ensemble.control import main
from ensemble.control.config_set import config_set
from ensemble.control.tests.test_status import StatusTestBase


class ControlEnsembleSetTest(StatusTestBase):

    @inlineCallbacks
    def setUp(self):
        yield super(ControlEnsembleSetTest, self).setUp()
        self.stderr = self.capture_stream("stderr")

    @inlineCallbacks
    def test_set_and_get(self):
        system = yield self.build_topology()

        self.mock_environment()
        self.mocker.replay()

        yield config_set(self.environment,
                         "wordpress",
                         ["blog-title=That\'ll do, pig."])

        # Verify the state is accessible
        wordpress = system["services"]["wordpress"]
        state = yield wordpress.get_config()
        self.assertEqual(state, {"blog-title": "That\'ll do, pig."})

    @inlineCallbacks
    def test_set_invalid_option(self):
        yield self.build_topology()
        self.mock_environment()
        finished = self.setup_cli_reactor()
        self.setup_exit(0)
        self.mocker.replay()

        main(["set",
              "wordpress",
              "blog-roll=What's a blog-roll?"])
        yield finished

        # Make sure we got an error message to the user
        self.assertIn("blog-roll is not a valid configuration option.",
                      self.stderr.getvalue())

    @inlineCallbacks
    def test_set_invalid_service(self):
        yield self.build_topology()
        self.mock_environment()
        finished = self.setup_cli_reactor()
        self.setup_exit(0)
        self.mocker.replay()

        main(["set",
              "whatever",
              "blog-roll=What's a blog-roll?"])
        yield finished

        self.assertIn("Service 'whatever' was not found",
                      self.stderr.getvalue())

    @inlineCallbacks
    def test_set_valid_option(self):
        system = yield self.build_topology()
        self.mock_environment()
        finished = self.setup_cli_reactor()
        self.setup_exit(0)
        self.mocker.replay()

        main(["set",
              "wordpress",
              'blog-title=My title'])
        yield finished

        # Verify the state is accessible
        wordpress = system["services"]["wordpress"]
        state = yield wordpress.get_config()
        self.assertEqual(state, {"blog-title": "My title"})
