from ensemble.formula.errors import (
    FormulaIdError, FormulaNotFound, InvalidFormulaHook, NewerFormulaNotFound)
from ensemble.errors import FormulaError

from ensemble.lib.testing import TestCase


class FormulaErrorsTest(TestCase):

    def test_NewerFormulaNotFound(self):
        error = NewerFormulaNotFound("local:name:21")
        self.assertEquals(
            str(error),
            "Formula 'local:name:21' is the latest revision known")
        self.assertTrue(isinstance(error, FormulaError))

    def test_FormulaIdError(self):
        error = FormulaIdError("foobar:/adfsa:slashot", "bad magic")
        self.assertEquals(
            str(error),
            "Invalid formula id 'foobar:/adfsa:slashot' bad magic")
        self.assertTrue(isinstance(error, FormulaError))

    def test_FormulaNotFound(self):
        error = FormulaNotFound("/path", "cassandra")
        self.assertEquals(
            str(error),
            "Formula 'cassandra' not found in repository /path")
        self.assertTrue(isinstance(error, FormulaError))

    def test_InvalidFormulaHook(self):
        error = InvalidFormulaHook("mysql", "magic-relation-changed")
        self.assertEquals(
            str(error),
            "Formula 'mysql' does not contain hook 'magic-relation-changed'")
        self.assertTrue(isinstance(error, FormulaError))
