from twisted.internet.defer import fail

from ensemble.errors import ProviderInteractionError

from .iterate import EC2ProviderMachineFilter
from .operation import EC2Operation
from .machine import EC2ProviderMachine


class EC2MachineAccessor(EC2Operation, EC2ProviderMachineFilter):
    """Retrieve a specific machine by provider id."""

    def run(self, provider_id):
        d = self.ec2.describe_instances(provider_id)
        d.addCallback(self._filter_provider_machines)
        d.addCallback(self._create_provider_machine, provider_id)
        return d

    def _create_provider_machine(self, instances, provider_id):
        if not instances:
            return fail(
                ProviderInteractionError(
                    "Machine (id: %r) was not found" % provider_id))

        instance = instances.pop()
        return EC2ProviderMachine(instance.instance_id, instance.dns_name)
