from ensemble.errors import (
    EnsembleError, FileNotFound, FileAlreadyExists, InvalidEnsembleHeaderValue,
    NoConnection, InvalidHost, InvalidUser, ProviderError,
    ProviderInteractionError, CannotTerminateMachine)

from ensemble.lib.testing import TestCase


class ErrorsTest(TestCase):

    def assertIsEnsembleError(self, error):
        self.assertTrue(isinstance(error, EnsembleError),
                        "%s is not a subclass of EnsembleError" %
                        error.__class__.__name__)

    def test_FileNotFound(self):
        error = FileNotFound("/path")
        self.assertEquals(str(error), "File was not found: /path")
        self.assertIsEnsembleError(error)

    def test_FileAlreadyExists(self):
        error = FileAlreadyExists("/path")
        self.assertEquals(str(error),
                          "File already exists, won't overwrite: /path")
        self.assertIsEnsembleError(error)

    def test_InvalidEnsembleHeaderValue(self):
        error = InvalidEnsembleHeaderValue("/path", "xpctd", "fnd")
        self.assertEquals(str(error),
                          "Expected a YAML file with an 'ensemble: xpctd' "
                          "header, found 'ensemble: fnd': /path")
        self.assertIsEnsembleError(error)

    def test_NoConnection(self):
        error = NoConnection("unable to connect")
        self.assertIsEnsembleError(error)

    def test_InvalidHost(self):
        error = InvalidHost("Invalid host for SSH forwarding")
        self.assertTrue(isinstance(error, NoConnection))
        self.assertEquals(
            str(error),
            "Invalid host for SSH forwarding")

    def test_InvalidUser(self):
        error = InvalidUser("Invalid SSH key")
        self.assertTrue(isinstance(error, NoConnection))
        self.assertEquals(
            str(error),
            "Invalid SSH key")

    def test_ProviderError(self):
        error = ProviderError("Invalid credentials")
        self.assertIsEnsembleError(error)

    def test_ProviderInteractionError(self):
        error = ProviderInteractionError(OSError("Bad Stuff"))
        self.assertIsEnsembleError(error)
        self.assertEquals(
            str(error),
            "ProviderError: Interaction with machine provider failed: "
            "OSError('Bad Stuff',)")

    def test_CannotTerminateMachine(self):
        error = CannotTerminateMachine(0, "environment would be destroyed")
        self.assertIsEnsembleError(error)
        self.assertEquals(
            str(error),
            "Cannot terminate machine 0: environment would be destroyed")
