
import os
import shutil

from twisted.internet.defer import inlineCallbacks, returnValue

from ensemble.formula.bundle import FormulaBundle

from ensemble.state.formula import FormulaStateManager
from ensemble.state.environment import EnvironmentStateManager


@inlineCallbacks
def download_formula(client, formula_id, formulas_directory):

    formula_state_manager = FormulaStateManager(client)
    formula_state = yield formula_state_manager.get_formula_state(formula_id)

    # XXX this is a temporary hack, in future we should use signed urls
    # directly to the formula and bypass the provider file storage.
    #
    # Its a hack because it requires access to the environment/provider
    # credentials, which should be restricted to the provisioning agent.
    #
    # https://bugs.launchpad.net/ensemble/+bug/761053

    environment_state_manager = EnvironmentStateManager(client)
    config = yield environment_state_manager.get_config()
    provider = config.get_default().get_machine_provider()
    file_storage = provider.get_file_storage()

    checksum = yield formula_state.get_sha256()
    formula_key = "%s:%s" % (formula_state.id, checksum)
    local_formula_path = os.path.join(
        formulas_directory, formula_key)

    remote_formula_path = "formulas/%s" % formula_key

    file_handle = yield file_storage.get(remote_formula_path)
    shutil.copyfileobj(
        file_handle, open(local_formula_path, "w"))

    returnValue(FormulaBundle(local_formula_path))
